@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2014 openpyxl
+# Copyright (c) 2010-2016 openpyxl
 
 """Read an xlsx file into Python"""
 
@@ -21,7 +21,7 @@ except ImportError:
 
 
 # package imports
-from openpyxl.exceptions import OpenModeError, InvalidFileException
+from openpyxl.utils.exceptions import InvalidFileException
 from openpyxl.xml.constants import (
     ARC_SHARED_STRINGS,
     ARC_CORE,
@@ -30,28 +30,33 @@ from openpyxl.xml.constants import (
     ARC_THEME,
     SHARED_STRINGS,
     EXTERNAL_LINK,
+    XLTM,
+    XLTX,
 )
 
-from openpyxl.workbook import Workbook, DocumentProperties
+from openpyxl.workbook import Workbook
 from openpyxl.workbook.names.external import detect_external_links
 from openpyxl.workbook.names.named_range import read_named_ranges
-from openpyxl.reader.strings import read_string_table
-from openpyxl.reader.style import read_style_table
-from openpyxl.reader.workbook import (
+from openpyxl.packaging.relationship import get_dependents
+from .strings import read_string_table
+from .style import read_style_table
+from .workbook import (
     read_content_types,
-    read_properties_core,
     read_excel_base_date,
     detect_worksheets,
     read_rels,
     read_workbook_code_name,
+    read_workbook_settings,
 )
-from openpyxl.reader.worksheet import read_worksheet
-from openpyxl.reader.comments import read_comments, get_comments_file
+from openpyxl.workbook.properties import read_properties, DocumentProperties
+from openpyxl.worksheet.read_only import ReadOnlyWorksheet
+from .worksheet import WorkSheetParser
+from openpyxl.comments.reader import read_comments, get_comments_file
 # Use exc_info for Python 2 compatibility with "except Exception[,/ as] e"
 
 
 CENTRAL_DIRECTORY_SIGNATURE = b'\x50\x4b\x05\x06'
-SUPPORTED_FORMATS = ('.xlsx', '.xlsm')
+SUPPORTED_FORMATS = ('.xlsx', '.xlsm', '.xltx', '.xltm')
 
 
 def repair_central_directory(zipFile, is_file_instance):
@@ -59,7 +64,7 @@ def repair_central_directory(zipFile, is_file_instance):
     code taken from http://stackoverflow.com/a/7457686/570216, courtesy of Uri Cohen
     '''
 
-    f = zipFile if is_file_instance else open(zipFile, 'r+b')
+    f = zipFile if is_file_instance else open(zipFile, 'rb+')
     data = f.read()
     pos = data.find(CENTRAL_DIRECTORY_SIGNATURE)  # End of central directory signature
     if (pos > 0):
@@ -73,6 +78,47 @@ def repair_central_directory(zipFile, is_file_instance):
     return f
 
 
+
+def _validate_archive(filename):
+    """
+    Check the file is a valid zipfile
+    """
+    is_file_like = hasattr(filename, 'read')
+
+    if not is_file_like and os.path.isfile(filename):
+        file_format = os.path.splitext(filename)[-1]
+        if file_format not in SUPPORTED_FORMATS:
+            if file_format == '.xls':
+                msg = ('openpyxl does not support the old .xls file format, '
+                       'please use xlrd to read this file, or convert it to '
+                       'the more recent .xlsx file format.')
+            elif file_format == '.xlsb':
+                msg = ('openpyxl does not support binary format .xlsb, '
+                       'please convert this file to .xlsx format if you want '
+                       'to open it with openpyxl')
+            else:
+                msg = ('openpyxl does not support %s file format, '
+                       'please check you can open '
+                       'it with Excel first. '
+                       'Supported formats are: %s') % (file_format,
+                                                       ','.join(SUPPORTED_FORMATS))
+            raise InvalidFileException(msg)
+
+
+    if is_file_like:
+        # fileobject must have been opened with 'rb' flag
+        # it is required by zipfile
+        if getattr(filename, 'encoding', None) is not None:
+            raise IOError("File-object must be opened in binary mode")
+
+    try:
+        archive = ZipFile(filename, 'r', ZIP_DEFLATED)
+    except BadZipfile:
+        f = repair_central_directory(filename, is_file_like)
+        archive = ZipFile(f, 'r', ZIP_DEFLATED)
+    return archive
+
+
 def load_workbook(filename, read_only=False, use_iterators=False, keep_vba=KEEP_VBA, guess_types=False, data_only=False):
     """Open the given filename and return the workbook
 
@@ -102,64 +148,14 @@ def load_workbook(filename, read_only=False, use_iterators=False, keep_vba=KEEP_
         and the returned workbook will be read-only.
 
     """
-
-    is_file_instance = isinstance(filename, file)
-
+    archive = _validate_archive(filename)
     read_only = read_only or use_iterators
 
-    if is_file_instance:
-        # fileobject must have been opened with 'rb' flag
-        # it is required by zipfile
-        if 'b' not in filename.mode:
-            raise OpenModeError("File-object must be opened in binary mode")
-
-    try:
-        archive = ZipFile(filename, 'r', ZIP_DEFLATED)
-    except BadZipfile:
-        file_format = os.path.splitext(filename)[-1]
-        if file_format not in SUPPORTED_FORMATS:
-            if file_format == '.xls':
-                msg = ('openpyxl does not support the old .xls file format, '
-                       'please use xlrd to read this file, or convert it to '
-                       'the more recent .xlsx file format.')
-            elif file_format == '.xlsb':
-                msg = ('openpyxl does not support binary format .xlsb, '
-                       'please convert this file to .xlsx format if you want '
-                       'to open it with openpyxl')
-            else:
-                msg = ('openpyxl does not support %s file format, '
-                       'please check you can open '
-                       'it with Excel first. '
-                       'Supported formats are: %s') % (file_format,
-                                                     ','.join(SUPPORTED_FORMATS))
-            raise InvalidFileException(msg)
-
-        try:
-            f = repair_central_directory(filename, is_file_instance)
-            archive = ZipFile(f, 'r', ZIP_DEFLATED)
-        except BadZipfile:
-            e = exc_info()[1]
-            raise InvalidFileException(unicode(e))
-    except (BadZipfile, RuntimeError, IOError, ValueError):
-        e = exc_info()[1]
-        raise InvalidFileException(unicode(e))
     wb = Workbook(guess_types=guess_types, data_only=data_only, read_only=read_only)
 
     if read_only and guess_types:
         warnings.warn('Data types are not guessed when using iterator reader')
 
-    try:
-        _load_workbook(wb, archive, filename, read_only, keep_vba)
-    except KeyError:
-        e = exc_info()[1]
-        raise InvalidFileException(unicode(e))
-
-    archive.close()
-    return wb
-
-
-def _load_workbook(wb, archive, filename, read_only, keep_vba):
-
     valid_files = archive.namelist()
 
     # If are going to preserve the vba then attach a copy of the archive to the
@@ -181,14 +177,13 @@ def _load_workbook(wb, archive, filename, read_only, keep_vba):
 
     # get workbook-level information
     try:
-        wb.properties = read_properties_core(archive.read(ARC_CORE))
+        wb.properties = read_properties(archive.read(ARC_CORE))
     except KeyError:
         wb.properties = DocumentProperties()
-    wb._read_workbook_settings(archive.read(ARC_WORKBOOK))
+    wb.active = read_workbook_settings(archive.read(ARC_WORKBOOK)) or 0
 
     # what content types do we have?
     cts = dict(read_content_types(archive))
-    rels = dict
 
     strings_path = cts.get(SHARED_STRINGS)
     if strings_path is not None:
@@ -198,39 +193,54 @@ def _load_workbook(wb, archive, filename, read_only, keep_vba):
     else:
         shared_strings = []
 
+    wb.is_template = XLTX in cts or XLTM in cts
+
     try:
         wb.loaded_theme = archive.read(ARC_THEME)  # some writers don't output a theme, live with it (fixes #160)
     except KeyError:
         assert wb.loaded_theme == None, "even though the theme information is missing there is a theme object ?"
 
-    style_table, color_index, cond_styles = read_style_table(archive.read(ARC_STYLE))
-    wb.shared_styles = style_table
-    wb.style_properties = {'dxf_list':cond_styles}
-    wb.cond_styles = cond_styles
+    parsed_styles = read_style_table(archive)
+    if parsed_styles is not None:
+        wb._differential_styles = parsed_styles.differential_styles
+        wb._cell_styles = parsed_styles.cell_styles
+        wb._named_styles = parsed_styles.named_styles
+        wb._colors = parsed_styles.color_index
+        wb._borders = parsed_styles.border_list
+        wb._fonts = parsed_styles.font_list
+        wb._fills = parsed_styles.fill_list
+        wb._number_formats = parsed_styles.number_formats
+        wb._protections = parsed_styles.protections
+        wb._alignments = parsed_styles.alignments
+        wb._colors = parsed_styles.color_index
 
-    wb.properties.excel_base_date = read_excel_base_date(xml_source=archive.read(ARC_WORKBOOK))
+    wb.excel_base_date = read_excel_base_date(archive)
 
     # get worksheets
-    wb.worksheets = []  # remove preset worksheet
+    wb._sheets = []  # remove preset worksheet
     for sheet in detect_worksheets(archive):
         sheet_name = sheet['title']
         worksheet_path = sheet['path']
         if not worksheet_path in valid_files:
             continue
 
-        if not read_only:
-            new_ws = read_worksheet(archive.read(worksheet_path), wb,
-                                    sheet_name, shared_strings, style_table,
-                                    color_index=color_index,
-                                    keep_vba=keep_vba)
+        if read_only:
+            new_ws = ReadOnlyWorksheet(wb, sheet_name, worksheet_path, None,
+                                       shared_strings)
+            wb._add_sheet(new_ws)
         else:
-            new_ws = read_worksheet(None, wb, sheet_name, shared_strings,
-                                    style_table,
-                                    color_index=color_index,
-                                    worksheet_path=worksheet_path)
+            fh = archive.open(worksheet_path)
+            parser = WorkSheetParser(wb, sheet_name, fh, shared_strings)
+            parser.parse()
+            new_ws = wb[sheet_name]
+        new_ws.sheet_state = sheet['state']
 
-        new_ws.sheet_state = sheet.get('state') or 'visible'
-        wb._add_sheet(new_ws)
+        if wb.vba_archive is not None and new_ws.legacy_drawing is not None:
+            # We need to get the file name of the legacy drawing
+            dirname, basename = worksheet_path.rsplit('/', 1)
+            rels_path = '/'.join((dirname, '_rels', basename + '.rels'))
+            rels = get_dependents(archive, rels_path)
+            new_ws.legacy_drawing = rels[new_ws.legacy_drawing].target
 
         if not read_only:
         # load comments into the worksheet cells
@@ -238,6 +248,7 @@ def _load_workbook(wb, archive, filename, read_only, keep_vba):
             if comments_file is not None:
                 read_comments(new_ws, archive.read(comments_file))
 
+    wb._differential_styles = [] # reset
     wb._named_ranges = list(read_named_ranges(archive.read(ARC_WORKBOOK), wb))
 
     wb.code_name = read_workbook_code_name(archive.read(ARC_WORKBOOK))
@@ -245,3 +256,7 @@ def _load_workbook(wb, archive, filename, read_only, keep_vba):
     if EXTERNAL_LINK in cts:
         rels = read_rels(archive)
         wb._external_links = list(detect_external_links(rels, archive))
+
+
+    archive.close()
+    return wb
